@@ -21,6 +21,8 @@ sub parse_arg {
 
 sub lock { $_[0]->lock_none }
 
+# XXX: need to handle peg revisions, ie
+# -r N PATH@M means the node PATH@M at rev N
 sub run {
     my ($self, $target, $target2) = @_;
     my $fs = $target->{repos}->fs;
@@ -41,24 +43,25 @@ sub run {
 	}
     }
     else {
-	delete $target->{copath} if $r1 && $r2;
+	$target->depotpath if $r1 && $r2;
 	if ($target->{copath}) {
-	    %$target2 = %$target;
-	    delete $target->{copath};
+	    $target2 = SVK::Target->new (%$target);
+	    $target->depotpath;
 	    $report = $target->{report};
 	}
 	else {
 	    # XXX: require revspec;
-	    %$target2 = %$target;
+	    $target2 = SVK::Target->new (%$target);
 	}
     }
 
     if ($target2->{copath}) {
-	$newroot = $self->{xd}->xdroot (%$target2);
+	$newroot = $target2->root ($self->{xd});
 	$oldroot = $newroot unless $r1;
+	my $lrev = $r1; # for the closure
 	$cb_llabel =
 	    sub { my ($rpath) = @_;
-		  'revision '.($r1 ||
+		  'revision '.($lrev ||
 			       $self->{xd}{checkout}->get ("$target2->{copath}/$rpath")->{revision});
 	      },
     }
@@ -83,6 +86,9 @@ sub run {
 	  $target->{path} ne $target2->{path} ?
 	  ( lpath  => $target->{path},
 	    rpath  => $target2->{path} ) : (),
+	  # XXX: for delete_entry, clean up these
+	  xd => $self->{xd}, newtarget => $target2,
+	  oldtarget => $target, oldroot => $oldroot,
 	);
 
     if ($target2->{copath}) {
@@ -136,15 +142,15 @@ SVK::Command::Diff - Display diff between revisions or checkout copies
 
 =head1 SYNOPSIS
 
-    diff [-r REV] [PATH]
-    diff -r N:M DEPOTPATH
-    diff DEPOTPATH1 DEPOTPATH2
-    diff DEPOTPATH PATH
+ diff [-r REV] [PATH]
+ diff -r N:M DEPOTPATH
+ diff DEPOTPATH1 DEPOTPATH2
+ diff DEPOTPATH PATH
 
 =head1 OPTIONS
 
-  -r [--revision] rev|old:new :	Needs description
-  -v [--verbose]:	Needs description
+ -r [--revision] rev|old:new :    Needs description
+ -v [--verbose]:                  Needs description
 
 =head1 AUTHORS
 
