@@ -8,7 +8,7 @@ use SVK::Util qw(get_buffer_from_editor);
 
 sub parse_arg {
     my ($self, @arg) = @_;
-    $self->usage unless $#arg == 1;
+    return unless $#arg == 1;
     return ($arg[0], $self->arg_co_maybe ($arg[1]));
 }
 
@@ -20,9 +20,7 @@ sub lock {
 sub run {
     my ($self, $pname, $target) = @_;
 
-    my $pvalue = $self->{xd}->do_proplist
-	(%$target, rev => $target->{repos}->fs->youngest_rev)->{$pname};
-
+    my $pvalue = $self->{xd}->do_proplist ($target)->{$pname};
     $pvalue = get_buffer_from_editor ("property $pname", undef, $pvalue || '',
 				      'prop');
 
@@ -41,14 +39,14 @@ SVK::Command::Propedit - Edit a property on path
 
 =head1 SYNOPSIS
 
-    propedit PROPNAME [PATH|DEPOTPATH...]
+ propedit PROPNAME [PATH|DEPOTPATH...]
 
 =head1 OPTIONS
 
-  -m [--message] message:	Commit message
-  -C [--check-only]:	Needs description
-  -s [--sign]:	Needs description
-  --force:	Needs description
+ -m [--message] message:    Commit message
+ -C [--check-only]:         Needs description
+ -s [--sign]:               Needs description
+ --force:                   Needs description
 
 =head1 AUTHORS
 
