@@ -104,21 +104,21 @@ sub apply_textdelta {
     return if $self->{check_only};
     my $copath = $path;
     $self->{get_copath}($copath);
+    my $dpath = $self->{anchor} eq '/' ? "/$path" : "$self->{anchor}/$path";
     if (-e $copath) {
 	my ($dir,$file) = get_anchor (1, $copath);
 	my $basename = "$dir.svk.$file.base";
-	$base = SVK::XD::get_fh ($self->{oldroot}, '<',
-				 "$self->{anchor}/$path", $copath);
+	$base = SVK::XD::get_fh ($self->{oldroot}, '<', $dpath, $copath);
 	if ($checksum) {
 	    my $md5 = md5($base);
 	    die loc("source checksum mismatch") if $md5 ne $checksum;
 	    seek $base, 0, 0;
 	}
 	rename ($copath, $basename);
-	$self->{base}{$path} = [$base, $basename];
+	$self->{base}{$path} = [$base, $basename,
+				-l $basename ? () : [stat($base)]];
     }
-    my $fh = SVK::XD::get_fh ($self->{newroot}, '>',
-			      "$self->{anchor}/$path", $copath)
+    my $fh = SVK::XD::get_fh ($self->{newroot}, '>', $dpath, $copath)
 	or warn "can't open $path";
 
     # The fh is refed by the current default pool, not the pool here
@@ -130,15 +130,15 @@ sub close_file {
     my ($self, $path) = @_;
     my $copath = $path;
     $self->{get_copath}($copath);
-    if ($self->{base}{$path}) {
-	chmod ((stat ($self->{base}{$path}[0]))[2], $copath);
-	close $self->{base}{$path}[0];
-	unlink $self->{base}{$path}[1];
+    if ((my $base = $self->{base}{$path})) {
+	close $base->[0];
+	unlink $base->[1];
+	chmod $base->[2][2], $copath if $base->[2];
 	delete $self->{base}{$path};
     }
     elsif (!$self->{update} && !$self->{check_only}) {
 	my $report = $path;
-	$report =~ s/^\Q$self->{target}\E/$self->{report}/;
+	$report =~ s/^\Q$self->{target}\E/$self->{report}/ if $self->{report};
 	$self->{xd}->do_add (report => $report,
 			     copath => $copath, quiet => $self->{quiet});
     }
@@ -153,9 +153,10 @@ sub add_directory {
     my ($self, $path) = @_;
     my $copath = $path;
     $self->{get_copath}($copath);
+    die loc("path %1 already exists", $copath) if -e $copath;
     mkdir ($copath) unless $self->{check_only};
     my $report = $path;
-    $report =~ s/^\Q$self->{target}\E/$self->{report}/;
+    $report =~ s/^\Q$self->{target}\E/$self->{report}/ if $self->{report};
     $self->{xd}->do_add (report => $report,
 			 copath => $copath, quiet => $self->{quiet})
 	if !$self->{update} && !$self->{check_only};
