@@ -31,12 +31,20 @@ sub skip_print {
     print "    ", loc("%1 - skipped\n", $path);
 }
 
-sub flush_print_report {
-    my $report = shift;
-    return \&flush_print unless defined $report;
+sub print_report {
+    my ($print, $report, $target) = @_;
+    return $print unless defined $report;
     sub {
-	my $path = $_[0] ? "$report$_[0]" : '.';
-	flush_print ($path, $_[1]);
+	my $path = shift;
+	if ($target) {
+	    if ($target eq $path) {
+		$path = '';
+	    }
+	    else {
+		$path =~ s|^\Q$target/||;
+	    }
+	}
+	$print->($path ? "$report$path" : $report || '.', @_);
     };
 }
 
@@ -47,6 +55,12 @@ sub new {
     return $self;
 }
 
+sub new_with_report {
+    my ($class, $report, $target) = @_;
+    $class->new	( cb_skip => print_report (\&skip_print, $report),
+		  cb_flush => print_report (\&flush_print, $report, $target));
+}
+
 sub node_status {
     my ($self, $path, $s) = @_;
     $self->{status}{$path}[0] = $s if defined $s;
